import React, {
  Component
} from 'react';
import {
  Container,
  Table,
  Form,
  FormGroup,
  Button,
  Input,
} from 'reactstrap';
import {
  FaTrashAlt,
  FaPlus
} from 'react-icons/fa';
import Navigation from './Navigation';
import Header from './Header';
import API from './Api';

class Properties extends Component {
  constructor(props) {
    super(props);

    this.getProperties = this.getProperties.bind(this);
    this.addProperty = this.addProperty.bind(this);
    this.deleteProperty = this.deleteProperty.bind(this);
    this.handleChange = this.handleChange.bind(this);

    this.state = {
      Properties: [],
      name: '',
    };

    this.getProperties();
  }

  handleChange(event) {
    const target = event.target;
    const value = target.type === 'checkbox' ? target.checked : target.value;
    const name = target.name;

    this.setState({
      [name]: value,
    });
  }

  getProperties() {
    API.get('properties')
      .then((res) => {
        if (res.status === 200) {
          this.setState({
            Properties: res.data.rows
          });
        } else {
          this.setState({
            Properties: []
          });
        }
      })
      .catch((e) => {
        alert('Erreur lors de la récupération des types de propriétés');
      });
  }

  addProperty(event) {
    event.preventDefault(); // Let's stop this event.
    event.stopPropagation(); // Really this time.
    API.post('properties', {
        name: this.state.name,
      })
      .then((res) => {
        this.setState(prevState => ({
          Properties: [...prevState.Properties, res.data],
          name: ''
        }));
      })
      .catch(() => {
        alert('Impossile de créer ce type de propriété');
      });
  }

  deleteProperty(property) {
    API.delete(`properties/${property.id}`)
      .then((res) => {
        this.setState({
          Properties: this.state.Properties.filter(item => item.id !== property.id),
        });
      })
      .catch((e) => {
        alert('Erreur lors de la suppression de ce type de propriété');
      });
  }

  render() {
    return (
      <div>
        <Header />
        <Container>
          <Navigation root="Property" />
          <Table>
            <thead>
              <tr>
                <th>#</th>
                <th>Nom</th>
                <th>Action</th>
              </tr>
            </thead>
            <tbody>
              {this.state.Properties.map((item, key) => (
                <tr key={key}>
                  <th scope="row">{item.id}</th>
                  <td>{item.name}</td>
                  <td>
                    {' '}
                    <Button color="danger" onClick={() => this.deleteProperty(item)}><FaTrashAlt /></Button>
                  </td>
                </tr>
              ))}
            </tbody>
          </Table>
          <Form inline onSubmit={this.addProperty}>
            <FormGroup className="mb-8 mr-sm-8 mb-sm-0">
              <Input
                type="text"
                name="name"
                value={this.state.name}
                placeholder="Nouveau type de propriété"
                onChange={this.handleChange}
              />
            </FormGroup>
            <Button color="primary">
              <FaPlus />
              {' '}
  Ajouter
            </Button>
          </Form>
        </Container>
      </div>
    );
  }
}

export default Properties;
